"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Context_1 = require("../ioc/Context");
const LogManager_1 = require("../log/LogManager");
const PreinstantiatedSingletonDefinition_1 = require("../ioc/objectdefinition/PreinstantiatedSingletonDefinition");
const Lifecycle_1 = require("../ioc/Lifecycle");
const ConfigProvider_1 = require("../config/ConfigProvider");
class BaseApp {
    /**
     * Creates a new Inceptum App
     */
    constructor(options = {}) {
        this.plugins = [];
        this.pluginContext = new Map();
        const { config = new ConfigProvider_1.default(), shutdownTimer = 100 } = options;
        const { logger = LogManager_1.LogManager.getLogger(__filename) } = options;
        this.logger = logger;
        this.logger.info(`Using app name ${LogManager_1.LogManager.getAppName()}`);
        this.context = new Context_1.Context(config.getConfig('app.context.name', 'BaseContext'), null, options);
        this.context.registerDefinition(new PreinstantiatedSingletonDefinition_1.PreinstantiatedSingletonDefinition(LogManager_1.LogManager));
        this.context.registerDefinition(new PreinstantiatedSingletonDefinition_1.PreinstantiatedSingletonDefinition(logger, 'logger'));
        this.context.on('STOPPED', () => LogManager_1.LogManager.scheduleShutdown());
    }
    use(...plugins) {
        return this.register(...plugins);
    }
    /**
     * Register services or controllers with Inceptum
     * Note that we are using "npm globby" internally for glob matching.
     *
     * Globs are disabled by default. Pass isGlob=true to enable glob matching
     *
     * Globs will be automatically activated if the "patterns" parameter is an array of strings,
     * or if it contains magic characters (eg: * ? { })
     *
     * @param {string|Array<string>} patterns - path as a relative path or as glob pattern(s). See options for more details
     * @param {Object} [options] - options object for enabling and configuring glob matching
     * @param {boolean} [options.isGlob=false] - pass true to treat the path as a glob
     * @param {Object} [options.globOptions] - options to pass to globby
     */
    addDirectory(patterns, options = {
        isGlob: false,
        globOptions: {},
    }) {
        return this.getContext().registerSingletonsInDir(patterns, options);
    }
    register(...plugins) {
        if (this.context.getStatus() !== Lifecycle_1.LifecycleState.NOT_STARTED) {
            throw new Error(`Cannot register plugin(s) ${plugins
                .map((p) => p.name)
                .join(',')} as the app has already started. Please register all plugins before calling "start()"`);
        }
        this.plugins = this.plugins.concat(plugins);
    }
    getRegisteredPluginNames() {
        return this.plugins.map((plugin) => plugin.name);
    }
    hasRegisteredPlugin(name) {
        return !!this.plugins.find((plugin) => plugin.name === name);
    }
    runLifecycleMethodOnPlugins(method) {
        return this.plugins.reduce(async (previous, plugin) => {
            await previous;
            if (plugin[method]) {
                this.logger.debug(`${method}:${plugin.name}`);
                return plugin[method](this, this.pluginContext);
            }
            return Promise.resolve();
        }, Promise.resolve());
    }
    async start() {
        await this.runLifecycleMethodOnPlugins('willStart');
        process.on('SIGINT', () => {
            this.stop().then(() => process.exit());
        });
        process.on('SIGTERM', () => {
            this.stop().then(() => process.exit());
        });
        await this.context.lcStart();
        return this.runLifecycleMethodOnPlugins('didStart');
    }
    async stop() {
        await this.runLifecycleMethodOnPlugins('willStop');
        this.logger.info('Shutting down app');
        await this.context.lcStop();
        const r = await this.runLifecycleMethodOnPlugins('didStop');
        delete this.logger;
        return r;
    }
    getContext() {
        return this.context;
    }
    // tslint:disable-next-line:prefer-function-over-method
    getConfig(key, defaultValue) {
        return this.getContext().getConfig(key, defaultValue);
    }
    // tslint:disable-next-line:prefer-function-over-method
    hasConfig(key) {
        return this.getContext().hasConfig(key);
    }
}
exports.default = BaseApp;
//# sourceMappingURL=BaseApp.js.map